#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <GL/glut.h>

#include "lib.h"
#include "Block.h"
#include "Piece.h"

struct _Piece {
  PieceType_t type;

  int count;

  vector_t rotate_angle;
  vector_t rotate_v;

  vector_t position;
  vector_t v;
  vector_t a;

  Block block;
};

Piece Piece_Create(Block block, PieceType_t type)
{
  Piece piece;

  piece = malloc(sizeof(*piece));
  memset(piece, 0, sizeof(*piece));

  piece->type = type;

  piece->count = rand() % 40 + 10;

  piece->rotate_angle.x = rand() % 360;
  piece->rotate_angle.y = rand() % 360;
  piece->rotate_angle.z = rand() % 360;
  piece->rotate_v.x = (rand() % 61) - 30;
  piece->rotate_v.y = (rand() % 61) - 30;
  piece->rotate_v.z = (rand() % 61) - 30;

  piece->position.x = 0.0;
  piece->position.y = 0.0;
  piece->position.z = 0.0;
  piece->v.x = ((rand() % 401) - 200) * 0.0001;
  piece->v.y = ((rand() % 401) - 200) * 0.0001;
  piece->v.z = ((rand() % 201) + 100) * 0.0001;
  piece->a.x = 0.0;
  piece->a.y = 0.0;
  piece->a.z = -0.001;

  piece->block = block;

  return piece;
}

Piece Piece_Destroy(Piece piece)
{
  if (piece) free(piece);
  return NULL;
}

int Piece_Proc(Piece piece)
{
  if (piece->count == 0)
    return 0;
  piece->count--;

  piece->rotate_angle.x += piece->rotate_v.x;
  piece->rotate_angle.y += piece->rotate_v.y;
  piece->rotate_angle.z += piece->rotate_v.z;

  piece->v.x += piece->a.x;
  piece->v.y += piece->a.y;
  piece->v.z += piece->a.z;

  piece->v.x *= 0.95;
  piece->v.y *= 0.95;

  piece->position.x += piece->v.x;
  piece->position.y += piece->v.y;
  piece->position.z += piece->v.z;

  return 1;
}

int Piece_MakePolygon(Piece piece)
{
  GLfloat red[]     = { 0.7, 0.0, 0.0, 0.6 };
  GLfloat magenta[] = { 0.7, 0.0, 0.7, 0.6 };
  GLfloat *color;

  switch (piece->type) {
  default:
  case PIECE_TYPE_FAIL:    color = magenta; break;
  case PIECE_TYPE_SUCCEED: color = red;     break;
  }

  glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT , color);
  glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE , color);
  glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, color);

  glTranslated(piece->position.x, piece->position.y, piece->position.z);
  glRotated(piece->rotate_angle.x, 1.0, 0.0, 0.0);
  glRotated(piece->rotate_angle.y, 0.0, 1.0, 0.0);
  glRotated(piece->rotate_angle.z, 0.0, 0.0, 1.0);
  glScaled(0.02, 0.02, 0.02);

  glBegin(GL_POLYGON);
  glNormal3d( 0.0, 0.0, 1.0);
  glVertex3d( 1.0, 1.0, 0.0);
  glVertex3d(-1.0, 1.0, 0.0);
  glVertex3d(-1.0,-1.0, 0.0);
  glVertex3d( 1.0,-1.0, 0.0);
  glEnd();

  return 0;
}
