#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <GL/glut.h>

#include "lib.h"
#include "Obj.h"
#include "Block.h"
#include "Saucer.h"

struct _Saucer {
  vector_t position;

  int count;
  vector_t v;

  int track_num;
  int sector_num;

  ObjList blocks;
};

Saucer Saucer_Create(double x, double y, double z,
		     int track_num, int sector_num)
{
  Saucer saucer;
  Block block;
  ObjList sectors;
  int t, s;

  saucer = malloc(sizeof(*saucer));
  memset(saucer, 0, sizeof(*saucer));

  saucer->position.x = x;
  saucer->position.y = y;
  saucer->position.z = z;

  saucer->count = 0;
  saucer->v.x = 0.0;
  saucer->v.y = 0.0;
  saucer->v.z = 0.0;

  saucer->track_num  = track_num;
  saucer->sector_num = sector_num;

  saucer->blocks = ObjList_Create();

  for (t = 0; t < track_num; t++) {
    sectors = ObjList_Create();
    for (s = 0; s < sector_num; s++) {
      block = Block_Create(t, s);
      ObjList_InsertObjToEnd(sectors, block, (ObjDestructor)Block_Destroy);
    }
    ObjList_InsertObjToEnd(saucer->blocks, sectors,
			   (ObjDestructor)ObjList_Destroy);
  }

  return saucer;
}

int Saucer_Proc(Saucer saucer)
{
  Block block;
  ObjList sectors;
  ObjListData obj, obj2;

  if (saucer->count > 0) {
    saucer->count--;
    saucer->position.x += saucer->v.x;
    saucer->position.y += saucer->v.y;
    saucer->position.z += saucer->v.z;
  }

  for (obj = ObjList_GetStart(saucer->blocks);
       !ObjList_IsEndEdge(saucer->blocks, obj);
       obj = ObjListData_GetNext(obj)) {
    sectors = (ObjList)ObjListData_GetObj(obj);
    for (obj2 = ObjList_GetStart(sectors);
	 !ObjList_IsEndEdge(sectors, obj2);
	 obj2 = ObjListData_GetNext(obj2)) {
      block = (Block)ObjListData_GetObj(obj2);
      Block_Proc(block);
    }
  }

  return 1;
}

int Saucer_MakePolygon(Saucer saucer)
{
  Block block;
  ObjList sectors;
  ObjListData obj, obj2;

  glTranslated(saucer->position.x, saucer->position.y, saucer->position.z);
  glScaled(0.5, 0.5, 0.5);

  for (obj = ObjList_GetStart(saucer->blocks);
       !ObjList_IsEndEdge(saucer->blocks, obj);
       obj = ObjListData_GetNext(obj)) {
    sectors = (ObjList)ObjListData_GetObj(obj);
    for (obj2 = ObjList_GetStart(sectors);
	 !ObjList_IsEndEdge(sectors, obj2);
	 obj2 = ObjListData_GetNext(obj2)) {
      block = (Block)ObjListData_GetObj(obj2);
      glPushMatrix();
      Block_MakePolygon(block, saucer->track_num, saucer->sector_num);
      glPopMatrix();
    }
  }

  return 0;
}

Block Saucer_GetBlock(Saucer saucer, int track, int sector)
{
  Block block;
  ObjList sectors;
  ObjListData obj, obj2;

  for (obj = ObjList_GetStart(saucer->blocks);
       !ObjList_IsEndEdge(saucer->blocks, obj);
       obj = ObjListData_GetNext(obj)) {

    if (track > 0) {
      track--;
      continue;
    }

    sectors = (ObjList)ObjListData_GetObj(obj);
    for (obj2 = ObjList_GetStart(sectors);
	 !ObjList_IsEndEdge(sectors, obj2);
	 obj2 = ObjListData_GetNext(obj2)) {
      if (sector > 0) {
	sector--;
	continue;
      }
      block = (Block)ObjListData_GetObj(obj2);
      return block;
    }
  }

  return NULL;
}

int Saucer_GetTrackNum(Saucer saucer)
{
  return saucer->track_num;
}

int Saucer_GetSectorNum(Saucer saucer)
{
  return saucer->sector_num;
}

int Saucer_AddTrackNum(Saucer saucer, int n)
{
  Block block;
  ObjList sectors;
  int t, s;

  for (t = saucer->track_num; t < saucer->track_num + n; t++) {
    sectors = ObjList_Create();
    for (s = 0; s < saucer->sector_num; s++) {
      block = Block_Create(t, s);
      ObjList_InsertObjToEnd(sectors, block, (ObjDestructor)Block_Destroy);
    }
    ObjList_InsertObjToEnd(saucer->blocks, sectors,
			   (ObjDestructor)ObjList_Destroy);
  }
  saucer->track_num += n;

  return saucer->track_num;
}

int Saucer_AddSectorNum(Saucer saucer, int n)
{
  Block block;
  ObjList sectors;
  ObjListData obj;
  int t, s;

  t = 0;
  for (obj = ObjList_GetStart(saucer->blocks);
       !ObjList_IsEndEdge(saucer->blocks, obj);
       obj = ObjListData_GetNext(obj)) {
    sectors = (ObjList)ObjListData_GetObj(obj);
    for (s = saucer->sector_num; s < saucer->sector_num + n; s++) {
      block = Block_Create(t, s);
      ObjList_InsertObjToEnd(sectors, block, (ObjDestructor)Block_Destroy);
    }
    t++;
  }
  saucer->sector_num += n;

  return saucer->sector_num;
}

int Saucer_Move(Saucer saucer, int count, double x, double y, double z)
{
  saucer->count = count;
  saucer->v.x = (x - saucer->position.x) / count;
  saucer->v.y = (y - saucer->position.y) / count;
  saucer->v.z = (z - saucer->position.z) / count;
  return 0;
}
