#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <GL/glut.h>

#include "lib.h"
#include "Wall.h"

struct _Wall {
  double stand_angle;
  Block block;
};

Wall Wall_Create(Block block)
{
  Wall wall;

  wall = malloc(sizeof(*wall));
  memset(wall, 0, sizeof(*wall));

  wall->stand_angle = 90;
  wall->block = block;

  return wall;
}

Wall Wall_Destroy(Wall wall)
{
  if (wall) free(wall);
  return NULL;
}

int Wall_Proc(Wall wall)
{
  if (Block_GetStatus(wall->block) == BLOCK_STATUS_CLEARING) {
    if (wall->stand_angle == 0) {
      Block_SetStatus(wall->block, BLOCK_STATUS_CLEARED);
    } else {
      wall->stand_angle -= 10;
    }
  }
  return 1;
}

int Wall_MakePolygon(Wall wall, int track_num, int sector_num, int track)
{
  int i;
  vector_t p0, p1;
  double r;
  vector_t points[4];
  GLfloat red[]     = { 0.7, 0.0, 0.0, 0.6 };
  GLfloat green[]   = { 0.0, 0.7, 0.0, 0.6 };
  GLfloat blue[]    = { 0.0, 0.0, 0.7, 0.6 };
  GLfloat yellow[]  = { 0.7, 0.7, 0.0, 0.6 };
  GLfloat magenta[] = { 0.7, 0.0, 0.7, 0.6 };
  GLfloat *color;

  switch (Block_GetStatus(wall->block)) {
  default:
  case BLOCK_STATUS_UNCLEAR:  color = blue;    break;
  case BLOCK_STATUS_TRY:      color = magenta; break;
  case BLOCK_STATUS_ATTACK:   color = red;     break;
  case BLOCK_STATUS_CLEARING: color = yellow;  break;
  case BLOCK_STATUS_CLEARED:  color = green;   break;
  }

  glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT , color);
  glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE , color);
  glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, color);

  p0.x = cos(M_PI / sector_num) * (1.0 / (track_num + 1)) * (track + 2);
  p0.y = sin(M_PI / sector_num) * (1.0 / (track_num + 1)) * (track + 2);
  p1.x = cos(M_PI / sector_num) * (1.0 / (track_num + 1)) * (track + 1);
  p1.y = sin(M_PI / sector_num) * (1.0 / (track_num + 1)) * (track + 1);

  r = p0.x;

  points[0].x = 0.0;
  points[0].y = -p0.y;
  points[0].z = 0.0;

  points[1].x = 0.0;
  points[1].y = +p0.y;
  points[1].z = 0.0;

  points[2].x = p1.x - p0.x;
  points[2].y = +p1.y;
  points[2].z = 0.0;

  points[3].x = p1.x - p0.x;
  points[3].y = -p1.y;
  points[3].z = 0.0;

  glRotated(wall->stand_angle, 0.0, 1.0, 0.0);

  glBegin(GL_POLYGON);
  glNormal3d(0.0, 0.0, 1.0);
  for (i = 0; i < 4; i++) {
    glVertex3d(points[i].x, points[i].y, points[i].z);
  }
  glEnd();

  return 0;
}
