#!/usr/bin/perl

################################
# Original code for Make Sample.

$dir = ".";
$speed = 2;
$rate = 0.1;
$quiet = 0;

while (($_ = shift(@ARGV)) ne "") {
	if    ($_ eq "-dir"  ) { $dir   = shift(@ARGV); }
	elsif ($_ eq "-d"    ) { $dir   = shift(@ARGV); }
	elsif ($_ eq "-speed") { $speed = shift(@ARGV); }
	elsif ($_ eq "-s"    ) { $speed = shift(@ARGV); }
	elsif ($_ eq "-rate" ) { $rate  = shift(@ARGV); }
	elsif ($_ eq "-r"    ) { $rate  = shift(@ARGV); }
	elsif ($_ eq "-quiet") { $quiet  = 1;           }
	elsif ($_ eq "-q"    ) { $quiet  = 1;           }
}

@uid_list = user_list();
@qid_list = question_list();
%clear_list = ();

while (1) {
	foreach $uid (@uid_list) {
		foreach $qid (@qid_list) {
			$key = "${uid}/${qid}";
			if (rand(100) < $speed) {
				question_reply($uid, $qid, "Invalid");
				if (!$quiet) {
					print "REPLY $uid $qid\n";
				}
			}
			if (defined($clear_list{$key})) {
				next;
			}
			if (rand(100) < $rate) {
				question_clear($uid, $qid);
				if (!$quiet) {
					print "CLEAR $uid $qid\n";
				}
			}
		}
	}
	sleep(1);
}

################################
# From EasyCTF library.

sub user_list {
	my $uid;
	my $name;
	my $password;
	my @list;

	@list = ();

	open(PASSWORDFILE, "$dir/user/password.txt");
	while (<PASSWORDFILE>) {
		s/^\s+//;
		s/\s+$//;
		if (/^#/ || ($_ eq "")) {
			next;
		}
		($uid, $name, $password) = split('\s+');
		push(@list, $uid);
	}
	close(PASSWORDFILE);

	return @list;
}

sub user_get_uid {
	my $search_name;
	my $uid;
	my $name;
	my $password;

	($search_name) = @_;

	open(PASSWORDFILE, "$dir/user/password.txt");
	while (<PASSWORDFILE>) {
		s/^\s+//;
		s/\s+$//;
		if (/^#/ || ($_ eq "")) {
			next;
		}
		($uid, $name, $password) = split('\s+');
		if ($search_name ne $name) {
			next;
		}
		close(PASSWORDFILE);
		return $uid;
	}
	close(PASSWORDFILE);

	return "";
}

sub user_get_name {
	my $uid;
	my $name;
	my $password;

	($uid) = @_;
	($uid, $name, $password) = user_readfile($uid);
	return $name;
}

sub user_readfile {
	my $search_uid;
	my $uid;
	my $name;
	my $password;

	($search_uid) = @_;

	open(PASSWORDFILE, "$dir/user/password.txt");
	while (<PASSWORDFILE>) {
		s/^\s+//;
		s/\s+$//;
		if (/^#/ || ($_ eq "")) {
			next;
		}
		($uid, $name, $password) = split('\s+');
		if ($search_uid ne $uid) {
			next;
		}
		close(PASSWORDFILE);
		return ($uid, $name, $password);
	}
	close(PASSWORDFILE);

	return "";
}

sub question_list {
	my $qid;
	my $file;
	my @list;

	@list = ();

	open(QUESTIONLIST, "$dir/question/list.txt");
	while (<QUESTIONLIST>) {
		s/^\s+//;
		s/\s+$//;
		if (/^#/ || ($_ eq "")) {
			next;
		}
		($qid, $file) = split('\s+');
		push(@list, $qid);
	}
	close(QUESTIONLIST);

	return @list;
}

sub question_get_file {
	my $search_qid;
	my $qid;
	my $file;

	($search_qid) = @_;

	open(QUESTIONLIST, "$dir/question/list.txt");
	while (<QUESTIONLIST>) {
		s/^\s+//;
		s/\s+$//;
		if (/^#/ || ($_ eq "")) {
			next;
		}
		($qid, $file) = split('\s+');
		if ($search_qid ne $qid) {
			next;
		}
		close(QUESTIONLIST);
		return $file;
	}
	close(QUESTIONLIST);

	return "";
}

sub question_get_genre {
	my $qid;
	my $genre;
	my $score;
	my $title;
	my $question;
	my @answers;

	($qid) = @_;
	($genre, $score, $title, $question, @answers) = question_readfile($qid);
	return $genre;
}

sub question_get_score {
	my $qid;
	my $genre;
	my $score;
	my $title;
	my $question;
	my @answers;

	($qid) = @_;
	($genre, $score, $title, $question, @answers) = question_readfile($qid);
	return $score;
}

sub question_get_title {
	my $qid;
	my $genre;
	my $score;
	my $title;
	my $question;
	my @answers;

	($qid) = @_;
	($genre, $score, $title, $question, @answers) = question_readfile($qid);
	return $title;
}

sub question_readfile {
	my $qid;
	my $file;
	my $key;
	my $value;
	my $in_question;
	my $in_comment;
	my $genre;
	my $score;
	my $title;
	my $question;
	my @answers;

	($qid) = @_;

	$file = question_get_file($qid);
	if ($file eq "") {
		return ();
	}

	$in_question = 0;
	$in_comment = 0;

	open(FILE, "$dir/question/$file");
	while (<FILE>) {
		if ($in_question) {
			if (/^\s*QUESTION_END/) {
				$in_question = 0;
			} else {
				$question .= "$_\n";
			}
			next;
		}
		if ($in_comment) {
			if (/^\s*COMMENT_END/) {
				$in_comment = 0;
			}
			next;
		}

		s/^\s+//;
		s/\s+$//;

		if (/^#/ || ($_ eq "")) {
			next;
		}

		($key, $value) = /(\w+):\s+(.+)/;

		if (/^GENRE/) {
			$genre = $value;
		}
		if (/^SCORE/) {
			$score = $value;
		}
		if (/^TITLE/) {
			$title = $value;
		}
		if (/^ANSWER/) {
			push(@answers, $value);
		}
		if (/^QUESTION/) {
			$in_question = 1;
		}
		if (/^COMMENT/) {
			$in_comment = 1;
		}
	}
	close(FILE);

	return ($genre, $score, $title, $question, @answers);
}

sub question_reply {
	my $uid;
	my $qid;
	my $reply;
	my $t;
	($uid, $qid, $reply) = @_;

	if (!-d "$dir/result") {
		mkdir("$dir/result");
	}
	if (!-d "$dir/result/$uid") {
		mkdir("$dir/result/$uid");
	}

	$t = time();
	open(RESULTFILE, ">> $dir/result/$uid/${qid}-${t}.txt");
	print RESULTFILE "$reply\n";
	print RESULTFILE "$t\n";
	$t = localtime($t);
	print RESULTFILE "$t\n";
	close(RESULTFILE);
}

sub question_clear {
	my $uid;
	my $qid;
	my $t;
	($uid, $qid) = @_;

	if (!-d "$dir/result") {
		mkdir("$dir/result");
	}
	if (!-d "$dir/result/$uid") {
		mkdir("$dir/result/$uid");
	}

	open(RESULTFILE, "> $dir/result/$uid/${qid}.txt");
	$t = time();
	print RESULTFILE "$t\n";
	$t = localtime($t);
	print RESULTFILE "$t\n";
	close(RESULTFILE);
}

sub question_is_clear {
	my $uid;
	my $qid;
	($uid, $qid) = @_;

	if (-f "$dir/result/$uid/${qid}.txt") {
		return 1;
	}
	return 0;
}

sub clear_get_time {
	my $uid;
	my $qid;
	($uid, $qid) = @_;

	if (!question_is_clear($uid, $qid)) {
		return 0;
	}

	open(RESULTFILE, "$dir/result/$uid/${qid}.txt");
	$t = <RESULTFILE>;
	close(RESULTFILE);

	return $t + 0;
}

sub clear_list {
	my @qid_list;
	my $uid;
	my $qid;
	my @clear_list;

	($uid) = @_;

	@qid_list = question_list();
	@clear_list = ();

	foreach $qid (@qid_list) {
		if (question_is_clear($uid, $qid)) {
			push(@clear_list, $qid);
		}
	}

	return @clear_list;
}

sub user_total_score {
	my @qid_list;
	my $uid;
	my $qid;
	my $t;
	my $total_score = 0;

	($uid, $t) = @_;

	@qid_list = clear_list($uid);

	foreach $qid (@qid_list) {
		if (defined($t) && ($t > 0)) {
			if (clear_get_time($uid, $qid) > $t) {
				next;
			}
		}
		$total_score += question_get_score($qid);
	}

	return $total_score;
}
